/*
   Copyright 2007-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.singleapp;

import com.notuvy.singleapp.multiuser.MultiUserDefer;
import com.notuvy.singleapp.multiuser.MultiUserPreempt;

import java.io.Serializable;

/**
 * The trivial implementation of a single instance application.
 * An actual implementation would extend this class and override the stubbed methods.
 *
 * @author  murali
 */
public abstract class AbstractSingleApp implements SingleApp {

    //------------------------------------------------------------
    //- Class Variables


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private final int fPort;
    private final Strategy fStrategy;


    //------------------------------------------------------------
    //- Constructors

    protected AbstractSingleApp(int pPort, Strategy pStrategy) {
        fPort = pPort;
        fStrategy = pStrategy;
    }


    //------------------------------------------------------------
    //- Accessors


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions

    public AbstractSingleApp launch() {
        switch (fStrategy) {

            case SINGLE_USER_DEFER:
                new SingleAppEnforcer.Defer(fPort, this).attempt();
                break;

            case SINGLE_USER_PREEMPT:
                new SingleAppEnforcer.Preempt(fPort, this).attempt();
                break;

            case MULTI_USER_DEFER:
                new MultiUserDefer(fPort, this).attempt();
                break;

            case MULTI_USER_PREEMPT:
                new MultiUserPreempt(fPort, this).attempt();
                break;

        }
        return this;
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    public Serializable sendFromDeferred() {
        return null;
    }

    public void recvFromDeferred(Object pObject) {
    }

    public Serializable sendToPreemptor() {
        return null;
    }

    public void recvFromPreempted(Object pObject) {
    }

    public void gracefulExitForPreempt() {
    }

    public void forceExit() {
        System.exit(0);
    }


    //------------------------------------------------------------
    //- Inner Classes

    public enum Strategy {
        SINGLE_USER_DEFER,
        SINGLE_USER_PREEMPT,
        MULTI_USER_DEFER,
        MULTI_USER_PREEMPT
    }


    //------------------------------------------------------------
    //- Main



}
