package com.notuvy.singleapp;

import org.apache.log4j.Logger;

import java.io.Serializable;
import java.util.Date;

/**
 * Basic description
 *
 * @author murali
 */
public class DummyApp extends AbstractSingleApp {

    //------------------------------------------------------------
    //- Class Variables

    static {
        System.setProperty("log4j.configuration", "singleapp_log4j.xml");
    }

    protected static final Logger LOG = Logger.getLogger("com.notuvy.singleapp");


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private boolean alive = true;


    //------------------------------------------------------------
    //- Constructors

    private DummyApp(Strategy pStrategy) {
        super(39234, pStrategy);
    }


    //------------------------------------------------------------
    //- Gettors


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions


    //------------------------------------------------------------
    //- Class Interface Functions

    public void run() {
        LOG.info("Started: " + new Date());
        while (alive) {
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                // Ignore
            }
        }
        LOG.info("Ended: " + new Date());
    }

    @Override
    public void gracefulExitForPreempt() {
        alive = false;
    }

    @Override
    public Serializable sendFromDeferred() {
        return System.currentTimeMillis();
    }

    @Override
    public void recvFromDeferred(Object pObject) {
        Long token = (Long) pObject;
        LOG.info(String.format("Other deferred [%d]: [%s]", token, new Date()));
    }


    //------------------------------------------------------------
    //- Inner Classes


    //------------------------------------------------------------
    //- Main

    public static void main(String[] args) {
        try {
            DummyApp app = null;
            for (String arg : args) {
                if ("defer".equalsIgnoreCase(arg) || "d".equalsIgnoreCase(arg)) {
                    app = new DummyApp(Strategy.MULTI_USER_DEFER);
                } else if ("preempt".equalsIgnoreCase(arg) || "p".equalsIgnoreCase(arg)) {
                    app = new DummyApp(Strategy.MULTI_USER_PREEMPT);
                }
            }
            if (app == null) {
                LOG.warn("Usage: singleapp-dummy <defer|preempt>");
            } else {
                app.launch();
            }
        } catch (Throwable thr) {
            LOG.error("Fatal error", thr);
        }
    }


}
