package com.notuvy.singleapp.multiuser;

import org.apache.log4j.Logger;

import java.io.IOException;
import java.net.Socket;
import java.net.SocketException;

/**
 * This is the entity that listens for connections.
 * It will run in a separate thread (needed to process incoming connections independently).
 *
 * @author murali
 */
public class Broker implements Runnable {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger("com.notuvy.singleapp");

    public static Broker CURRENT = null;


    //------------------------------------------------------------
    //- Class Functions

    protected static void createWith(MultiUserSingleAppEnforcer pEnforcer) {
        if (CURRENT == null) {
            CURRENT = new Broker(pEnforcer);

            Thread thread = new Thread(CURRENT, "SingleAppBroker");
            thread.setDaemon(true);
            thread.start();
        } else {
            LOG.error("Another broker is already active so the new creation is rejected.");
        }
    }

    protected static void terminateCurrent() {
        if (CURRENT != null) {
            CURRENT.terminate();
        }
    }


    //------------------------------------------------------------
    //- Instance Variables

    private boolean fAlive = true;
    private final MultiUserSingleAppEnforcer fEnforcer;


    //------------------------------------------------------------
    //- Constructors

    private Broker(MultiUserSingleAppEnforcer pEnforcer) {
        fEnforcer = pEnforcer;
    }


    //------------------------------------------------------------
    //- Gettors


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    private void terminate() {
        fAlive = false;  // This will let the Thread end.
        CURRENT = null;
        try {
            LOG.debug("Terminating the SingleApp broker.");
            fEnforcer.getServer().close();
        } catch (IOException ioe) {
            LOG.error("Error closing server.", ioe);
        }
    }


    //------------------------------------------------------------
    //- Public Interface Functions


    //------------------------------------------------------------
    //- Class Interface Functions

    public void run() {
        while (fAlive) {
            try {
                Socket socket = fEnforcer.getServer().accept();
                fEnforcer.handleConnection(socket);
            } catch (SocketException se) {
                if (se.getMessage().equals("Socket closed")) {
                    // This is expected (not an error) when the server socket was manually closed via terminate().
                } else {
                    LOG.error("Problem with a the underlying connection.", se);
                }
            } catch (IOException ioe) {
                LOG.error("Problem with a new connection.", ioe);
            }
        }
    }


    //------------------------------------------------------------
    //- Inner Classes


    //------------------------------------------------------------
    //- Main


}
