package com.notuvy.singleapp.multiuser;

import com.notuvy.singleapp.SingleApp;

import java.io.*;
import java.net.Socket;

/**
 * Basic description
 *
 * @author murali
 */
public class MultiUserDefer extends MultiUserSingleAppEnforcer {

    //------------------------------------------------------------
    //- Class Variables


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables


    //------------------------------------------------------------
    //- Constructors

    public MultiUserDefer(int pPort, SingleApp pSingleApp) {
        super(pPort, pSingleApp);
    }


    //------------------------------------------------------------
    //- Gettors


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions


    //------------------------------------------------------------
    //- Class Interface Functions

    protected void handlePortConflict() {
        try {
            // Contact the broker and ask if this user already has an active instance.
            BrokerConnection brokerConnection = new BrokerConnection(getUser(), new Socket("localhost", getPort()));

            boolean conflict = brokerConnection.readUserConflict(getUser());
            if (conflict) {
                LOG.error("Not starting; another instance is already running for this user.");
                Serializable message = getSingleApp().sendFromDeferred();
                brokerConnection.writeObjectMessage(message);
                // After sending the message, this instance defers, meaning it will shut down, so close the connection.
                brokerConnection.closeByInput();
            } else {
                // No other instance is alive for this user, so go ahead and start.
                startBrokerClientListener(brokerConnection, new BrokerClientListener());

                getSingleApp().run();
            }
        } catch (IOException ioe) {
            LOG.error("Problem forwarding on [" + getPort() + "].", ioe);
        }
    }

    protected void handleConnection(Socket pSocket) {
        try {
            BrokerConnection incomingConnection = new BrokerConnection(pSocket);

            boolean conflict = alreadyHaveInstanceForUser(incomingConnection);

            incomingConnection.writeUserConflict(conflict);

            if (conflict) {
                Object message = incomingConnection.readObjectMessage();
                // The deferring instance will have died after sending the message, so close the connection.
                incomingConnection.closeByInput();

                BrokerConnection clientConnection = fUserMap.get(incomingConnection.getUser());
                if (clientConnection == null) {
                    // This is a request to the broker owner.
                    getSingleApp().recvFromDeferred(message);
                } else {
                    // This is a request to a broker client.  Forward the message.
                    clientConnection.getOutput().writeInt(MSG_TYPE_DEFER_MESSAGE);
                    clientConnection.writeObjectMessage(message);
                }
            } else {
                fUserMap.put(incomingConnection.getUser(), incomingConnection);
            }
        } catch (IOException ioe) {
            LOG.error("Could read forwarded data.", ioe);
        }
    }

    protected void handleRelayedMessage() throws IOException {
        Object message = fClientConnection.readObjectMessage();
        getSingleApp().recvFromDeferred(message);
    }


    //------------------------------------------------------------
    //- Inner Classes

    class BrokerClientListener implements Runnable {
        public void run() {
            try {
                while (true) {
                    int msgType = fClientConnection.getInput().readInt();
                    if (MSG_TYPE_PING == msgType) {
                        handlePing();
                    } else if (MSG_TYPE_DEFER_MESSAGE == msgType) {
                        handleRelayedMessage();
                    } else {
                        LOG.error(String.format("Unrecognized message type [%d].", msgType));
                    }
                }
            } catch (EOFException eofe) {
                LOG.debug("Connection to broker lost, this should become broker");
                reattempt();
            } catch (IOException ioe) {
                LOG.error("Could not processes forwarded message.", ioe);
            }
        }
    }


    //------------------------------------------------------------
    //- Main


}
